﻿using Breakout.NavigationExtensions.Models;
using Orchard.ContentManagement;
using Orchard.ContentManagement.Drivers;
using Orchard.ContentManagement.Handlers;
using Orchard.Environment;
using Orchard.Environment.Extensions;
using Orchard.Localization;
using Orchard.Widgets.Services;
using System;
using System.Linq;

namespace Breakout.NavigationExtensions.Drivers
{
    [OrchardFeature("Breakout.NavigationExtensions.MenuItemLayerRules")]
    public class MenuItemLayerRulePartDriver : ContentPartDriver<MenuItemLayerRulePart>
    {
        private readonly Work<IRuleManager> ruleManagerAccessor;

        public MenuItemLayerRulePartDriver(Work<IRuleManager> ruleManagerAccessor)
        {
            this.ruleManagerAccessor = ruleManagerAccessor;

            T = NullLocalizer.Instance;
        }

        public Localizer T { get; set; }

        protected override string Prefix
        {
            get { return "MenuItemLayerRulePart"; }
        }

        protected override DriverResult Editor(MenuItemLayerRulePart part, dynamic shapeHelper)
        {
            return ContentShape("Parts_ItemLayerRulePart_Edit",
                () => shapeHelper.EditorTemplate(
                    TemplateName: "Parts.ItemLayerRulePart.Edit",
                    Model: part,
                    Prefix: Prefix));
        }

        protected override DriverResult Editor(MenuItemLayerRulePart part, IUpdateModel updater, dynamic shapeHelper)
        {
            updater.TryUpdateModel(part, Prefix, null, null);

            try
            {
                var manager = ruleManagerAccessor.Value;
                manager.Matches(part.LayerRule);
            }
            catch (Exception e)
            {
                updater.AddModelError("Description", T("The rule is not valid: {0}", e.Message));
            }

            return Editor(part, shapeHelper);
        }

        protected override void Exporting(MenuItemLayerRulePart part, ExportContentContext context)
        {
            var element = context.Element(part.PartDefinition.Name);

            element.SetAttributeValue("Rule", part.LayerRule);
        }

        protected override void Importing(MenuItemLayerRulePart part, ImportContentContext context)
        {
            var partName = part.PartDefinition.Name;

            context.ImportAttribute(partName, "Rule", value => part.LayerRule = value);
        }
    }   


}