﻿using Orchard.ContentManagement.Handlers;
using Orchard.ContentManagement;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using Orchard.Core.Navigation.Services;
using Orchard.UI.Navigation;
using Orchard.Core.Navigation.Models;
using Orchard.Widgets.Models;
using Orchard.Environment.Extensions;

namespace Breakout.NavigationExtensions.Handlers
{
    [OrchardFeature("Breakout.NavigationExtensions.HideNavigationWidget")]
    public class HideMenuWidgetPartHandler : ContentHandler
    {
        /// <summary>
        /// When the navigation is loaded it checks the menu items. If there are no items it moves the widget shape from being rendered on a visible zone. 
        /// Effectively it will make the zone null if it is the only shape placed on it and therefore cause the layout not to render the section therefore no extra html. 
        /// </summary>
        /// <param name="menuService"></param>
        /// <param name="navigationManager"></param>
        public HideMenuWidgetPartHandler(IMenuService menuService, INavigationManager navigationManager)
        {
            OnGetDisplayShape<MenuWidgetPart>((context, part) =>
            {
                var menu = menuService.GetMenu(part.Menu.Id);
                var menuItems = navigationManager.BuildMenu(menu).ToList();

                if (menuItems.Count == 0)
                {
                    var widgetPart = part.As<WidgetPart>();
                    //place the widget somewhere it will not be rendered 
                    widgetPart.Zone = "None";
                }
            });
        }
    }
}