using Breakout.NavigationExtensions.Models;
using Breakout.NavigationExtensions.Models.Settings;
using Orchard.ContentManagement.MetaData;
using Orchard.Core.Contents.Extensions;
using System;

namespace Breakout.NavigationExtensions.MigrationTasks.MenuItemLayer
{
    internal static class ContentTypesAndParts
    {
        internal static void CreateNewMenuItemType(IContentDefinitionManager contentDefinitionManager)
        {
            contentDefinitionManager.AlterTypeDefinition(Definitions.CustomLinkWithRules, cfg => 
                cfg
                    .WithPart("MenuPart")
                    .WithPart("IdentityPart")
                    .WithPart("CommonPart")
                    .WithPart("MenuItemLayerRulePart")
                    .DisplayedAs("Custom Link with Layer rules")
                    .WithSetting("Description", "Represents a simple custom link with a text and an url, displayed if its layer rules match.")
                    .WithSetting("Stereotype", "MenuItem"));
        }

        internal static void AlterLayerPartConfiguration(IContentDefinitionManager contentDefinitionManager)
        {
            contentDefinitionManager.AlterPartDefinition(typeof(MenuItemLayerRulePart).Name, cfg => 
                cfg
                    .Attachable(true)
                //    .WithDescription("Menu item rules for filtering if the menu item is accessible - by its own layer rules")
                );
        }

        //still doesn't allow default settings shame
        internal static void AddDefaultSettingsForMenuItemLayerPart(IContentDefinitionManager contentDefinitionManager)
        {
            string settingsName = typeof(MenuItemLayeRulePartSettings).Name;
            Func<string, string> setting = (field) => string.Format("{0}.{1}", settingsName, field);

            contentDefinitionManager.AlterPartDefinition(Definitions.MenuItemLayerRulePart, cfg =>
                cfg
                    .WithSetting(setting("EnableRuleForMenuItem"), true.ToString())
                    .WithSetting(setting("EnableChildrenRuleForMenuItem"), true.ToString()));
             
        }
    }
}