﻿using Orchard;
using Orchard.UI.Navigation;
using Orchard.ContentManagement;
using Orchard.Widgets.Services;
using System;
using System.Linq;
using Breakout.NavigationExtensions.Models;
using Orchard.Environment.Extensions;
using Breakout.NavigationExtensions.Models.Settings;
using System.Collections.Generic;

namespace Breakout.NavigationExtensions.Services
{
    [OrchardFeature("Breakout.NavigationExtensions.MenuItemLayerRules")]
    public class MenuItemLayerRuleService : IMenuItemLayerRuleService
    {
        private readonly IRuleManager ruleManagerAccessor;
        private readonly IDictionary<string, bool> accessiblePaths = new Dictionary<string, bool>();
        private readonly IDictionary<string, bool> evaluated = new Dictionary<string, bool>();
        private readonly IDictionary<string, bool> denyDaughterPaths = new Dictionary<string, bool>();

        public MenuItemLayerRuleService(IRuleManager ruleManagerAccessor) 
        {
            this.ruleManagerAccessor = ruleManagerAccessor;
        }

        public bool EvaluateMenuItem(MenuItem menuItem)
        {
            var content = menuItem.Content;
            var path = menuItem.Position;
            if (!EvaluatePath(menuItem)) 
            {
                return false;
            }

            if (!content.Has<MenuItemLayerRulePart>()) 
            {
                return true; 
            }

            if (evaluated.ContainsKey(path)) { return evaluated[path]; }

            var layerPart = content.As<MenuItemLayerRulePart>();
            var partSettings = layerPart.Settings.GetModel<MenuItemLayeRulePartSettings>();

            if (!DisplayItem(layerPart, partSettings)) {
                accessiblePaths[menuItem.Position] = false;
                
                return false;
            }

            if (!DisplayItemChildren(layerPart, partSettings)) 
            {
                denyDaughterPaths.Add(menuItem.Position, true); 
                menuItem.Items = Enumerable.Empty<MenuItem>();
            }

            if (!evaluated.ContainsKey(path)) { evaluated.Add(path, true); }

            return true;
        }

        private bool EvaluatePathOf(string path, bool parent) 
        {
            var containsPath = accessiblePaths.ContainsKey(path);

            if (containsPath) 
            {
                return parent ? accessiblePaths[path] : parent;
            }

            accessiblePaths.Add(path, parent);

            return parent;
        }
        
        private bool EvaluatePath(MenuItem menuItem) 
        {
            var path = menuItem.Position;
            var positions = path.Split('.');
            var totalDepth = positions.Length;//path.Where(e => e == '.').Count();

            bool pathAllowable = true;
            bool denyDaughterPath = false;
            string currentPath = string.Empty;

            for (int depth = 0; depth < totalDepth; depth++) 
            {
                if (denyDaughterPath)
                    pathAllowable = false;

                var position = positions[depth];
                
                currentPath = string.IsNullOrEmpty(currentPath) ? position : string.Format("{0}.{1}", currentPath, position);

                pathAllowable = EvaluatePathOf(currentPath, pathAllowable);
                denyDaughterPath = denyDaughterPaths.ContainsKey(currentPath) ? denyDaughterPaths[currentPath] : false;
            }

            return pathAllowable;
        }

        private bool DisplayItem(MenuItemLayerRulePart layerPart, MenuItemLayeRulePartSettings settings) 
        {
            if (settings.DisableLayerRuleForMenuItem)
                return true;
            if (string.IsNullOrWhiteSpace(layerPart.LayerRule))
                return true;

            return ruleManagerAccessor.Matches(layerPart.LayerRule);
        }

        private bool DisplayItemChildren(MenuItemLayerRulePart layerPart, MenuItemLayeRulePartSettings settings)
        {
            if (settings.DisableLayerRuleForChildrenWithinMenuItem)
                return true;
            if (string.IsNullOrWhiteSpace(layerPart.ChildrenLayerRule))
                return true;

            return ruleManagerAccessor.Matches(layerPart.ChildrenLayerRule);
        }

       
    }
}